import set from 'lodash/set';
import negate from 'lodash/negate';
import isUndefined from 'lodash/isUndefined';
import pickBy from 'lodash/pickBy';
import forEach from 'lodash/forEach';
import toPlainObject from 'lodash/toPlainObject';
import find from 'lodash/fp/find';
import flow from 'lodash/fp/flow';
import filter from 'lodash/fp/filter';
import { mapObject } from './helper';

export const getKeyName = (meta) => {
  const keyProperty = flow(
    filter(property => !property.skip),
    find(property => property.key),
  )(meta.properties || []);
  return keyProperty ? keyProperty.name : undefined;
};

export const normMetas = (metas) => {
  const ret = {};
  if (!metas) {
    return ret;
  }
  forEach(metas, (value, key) => {
    let finalValue;
    try {
      finalValue = JSON.parse(value);
    } catch (err) {
      finalValue = value;
    }
    set(ret, key, finalValue);
  });
  return pickBy(ret, negate(isUndefined));
};

export const parseMetas = (datasource) => {
  const res = {
    global: {
      columnData: {},
      rowData: {},
    },
    properties: {},
  };
  // noinspection JSUnresolvedVariable
  if (datasource.metas) {
    res.global.columnData = normMetas(datasource.metas);
  }
  res.properties = {};
  (datasource.properties || [])
    .filter(property => !property.skip)
    .forEach((property) => {
      // noinspection JSUnresolvedVariable
      if (property.metas) {
        const metas = normMetas(property.metas);
        for (const key of Object.keys(metas)) {
          const trimedKey = key.trim();
          if (trimedKey.startsWith('meta:')) {
            const pos = trimedKey.indexOf('.');
            if (pos === -1) {
              const mName = trimedKey.substring(5).trimLeft();
              res.global.rowData[mName] = property.name;
            } else {
              const pName = trimedKey.substring(5, pos).trimLeft();
              const mName = trimedKey.substring(pos + 1).trimLeft();
              if (!res.properties[pName]) {
                res.properties[pName] = {
                  columnData: {},
                  rowData: {},
                };
              }
              res.properties[pName].rowData[mName] = property.name;
            }
            return;
          }
        }
        if (!res.properties[property.name]) {
          res.properties[property.name] = {
            columnData: {},
            rowData: {},
          };
          res.properties[property.name].columnData = metas;
        }
      } else {
        res.properties[property.name] = {
          columnData: {},
          rowData: {},
        };
      }
    });
  return res;
};

export const parseQueryResult = ({ dataType, arrayData, singularData }, meta) => {
  if (dataType === 'TABLE') {
    const data = (arrayData || []).map(() => ({}));
    (meta.properties || [])
      .filter(property => !property.skip)
      .forEach((property, i) => {
        data.forEach((row, j) => {
          row[property.name] = arrayData[j][i]; // eslint-disable-line no-param-reassign
        });
      });
    return data;
  } else if (dataType === 'PROPERTIES') {
    const data = [];
    (meta.properties || [])
      .filter(property => !property.skip)
      .forEach((property) => {
        data.push((singularData || {})[property.name]);
      });
    return [toPlainObject(data)];
  } else {
    throw new Error(`Unsupported data type: ${dataType}`);
  }
};

const combineMeta = ({ columnData, rowData }, row) => {
  return {
    ...columnData,
    ...mapObject(rowData, v => row[v]),
  };
};

export const calcPropertyMeta = (property, parsedMeta, row) => {
  return combineMeta(parsedMeta.properties[property], row);
};

export const calcGlobalMeta = (parsedMeta, row) => {
  return combineMeta(parsedMeta.global, row);
};

